
## It appears that ESRI stores float(single-precision floating point)
## and double (double-precision floating point)data values using scientific
## notation in the dbf files that make up a shapefile.
## The read.dbf function in foreign seems to be able to
## read the scientific notation, but the write.dbf function appears to
## rely on fixed point floating point numbers. As a result, write.dbf
## rounds small numbers to zero or truncates them. In addition, writing and then reading
## the shapefile back into R using writeSpatialShape gives different numeric values.

library(maptools)

setwd("d:/projects/ssnpackage/Rcode/subversion/subsetSSN/examples")

### Example 1 ####################################################################
edges1 <- readShapeSpatial("edges")

writeSpatialShape(edges1, "edges2")
edges2 <- readShapeSpatial("edges2")

#these values are slightly different
max(edges1@data$AFVslope - edges2@data$AFVslope)
##[1] 5.000002e-16

max(edges1@data$AFVslope[1] - edges2@data$AFVslope[1])
##[1] -3.2729e-16

rm(edges1, edges2)

### Example 2 ###################################################################
##------- Differences also occur when using read.dbf and write.dbf directly
dbf1 <- read.dbf("sites.dbf")
write.dbf(dbf1, "sites2.dbf")
dbf2 <- read.dbf("sites2.dbf")

max(dbf1$AFVslope - dbf2$AFVslope)
##[1] 5.000002e-16

dbf1$AFVslope[2] - dbf2$AFVslope[2]
##[1] 5.000002e-16

rm(dbf1,dbf2)


### Example 3 ###################################################################

dbf1 <- read.dbf("obs.dbf")
write.dbf(dbf1, "obs2.dbf")
dbf2 <- read.dbf("obs2.dbf")

max(dbf1$NEAR_DIST- dbf2$NEAR_DIST)
##[1] 4.913702e-16

rm(dbf1,dbf2)

### Example 4 ###################################################################

dbf1 <- read.dbf("preds.dbf")
write.dbf(dbf1, "preds2.dbf")
dbf2 <- read.dbf("preds2.dbf")

max(dbf1$NEAR_DIST- dbf2$NEAR_DIST)
##[1] 4.87e-16

rm(dbf1,dbf2)

#----------------------------------------------------------------------

### Example 5 ###################################################################
setwd("d:/projects/ssnpackage/Rcode/subversion/subsetSSN/examples/MiddleFork04.ssn")

dbf1 <- read.dbf("strms.dbf")
write.dbf(dbf1, "strms2.dbf")
dbf2 <- read.dbf("strms2.dbf")


max(dbf1$areaPI- dbf2$areaPI)
##[1] 5.000002e-16

rm(dbf1,dbf2)


### Example 6 ###################################################################

dbf1 <- read.dbf("pred1km.dbf")
write.dbf(dbf1, "pred1km2.dbf")
dbf2 <- read.dbf("pred1km2.dbf")


max(dbf1$afvArea- dbf2$afvArea)
##[1] 5.000002e-16

rm(dbf1,dbf2)



